@@ -1,5 +1,5 @@
 from clld.db.meta import Base
-from clld.db.meta import PolymorphicBaseMixin
+from clld.db.meta import PolymorphicBaseMixin, DBSession
 from clld.db.models import IdNameDescriptionMixin
 from clld.db.models import Sentence
 from clld.db.models.common import Contribution
@@ -15,8 +15,29 @@ from sqlalchemy import UniqueConstraint
 from sqlalchemy import Table
 from sqlalchemy.orm import relationship
 from zope.interface import implementer
-from clld_corpus_plugin.interfaces import IText
+from clld_corpus_plugin.interfaces import IText, ITag, ISpeaker
 from clld_corpus_plugin.interfaces import IWordform, IMeaning
+from sqlalchemy.ext.hybrid import hybrid_property
+from sqlalchemy import case, select, func
+from sqlalchemy.orm import object_session
+
+
+@implementer(ITag)
+class Tag(Base, IdNameDescriptionMixin):
+    pass
+
+
+@implementer(ISpeaker)
+class Speaker(Base, IdNameDescriptionMixin):
+    pass
+
+class SpeakerSentence(Base, PolymorphicBaseMixin):
+    __table_args__ = (UniqueConstraint("speaker_pk", "sentence_pk"),)
+
+    sentence_pk = Column(Integer, ForeignKey("sentence.pk"), nullable=False)
+    speaker_pk = Column(Integer, ForeignKey("speaker.pk"), nullable=False)
+    speaker = relationship(Speaker, innerjoin=True, backref="sentences")
+    sentence = relationship(Sentence, innerjoin=True, backref="speaker")    
 
 
 @implementer(IText)
@@ -24,20 +45,43 @@ class Text(Base, IdNameDescriptionMixin, HasSourceMixin):
     text_type = Column(Unicode())
     text_metadata = Column(JSON)
 
+    @property
+    def part_count(self):
+        return len(self.sentences)
+
+
+class TextTag(Base, PolymorphicBaseMixin):
+    __table_args__ = (UniqueConstraint("text_pk", "tag_pk"),)
+
+    text_pk = Column(Integer, ForeignKey("text.pk"), nullable=False)
+    tag_pk = Column(Integer, ForeignKey("tag.pk"), nullable=False)
+    text = relationship(Text, innerjoin=True, backref="tags")
+    tag = relationship(Tag, innerjoin=True, backref="texts")
+
+
+class SentenceTag(Base, PolymorphicBaseMixin):
+    __table_args__ = (UniqueConstraint("sentence_pk", "tag_pk"),)
+
+    sentence_pk = Column(Integer, ForeignKey("sentence.pk"), nullable=False)
+    tag_pk = Column(Integer, ForeignKey("tag.pk"), nullable=False)
+    sentence = relationship(Sentence, innerjoin=True, backref="tags")
+    tag = relationship(Tag, innerjoin=True, backref="sentences")
+
 
 try:
     from clld_morphology_plugin.models import Wordform, Meaning, FormMeaning
 except ImportError:
+
     @implementer(IWordform)
     class Wordform(Base, PolymorphicBaseMixin, IdNameDescriptionMixin, HasSourceMixin):
         __table_args__ = (UniqueConstraint("language_pk", "id"),)
-    
+
         language_pk = Column(Integer, ForeignKey("language.pk"), nullable=False)
         language = relationship(Language, innerjoin=True)
-    
+
         contribution_pk = Column(Integer, ForeignKey("contribution.pk"))
         contribution = relationship(Contribution, backref="wordforms")
-    
+
     @implementer(IMeaning)
     class Meaning(Base, PolymorphicBaseMixin, IdNameDescriptionMixin):
         pass
@@ -55,9 +99,11 @@ class TextSentence(Base, PolymorphicBaseMixin):
 
     text_pk = Column(Integer, ForeignKey("text.pk"), nullable=False)
     sentence_pk = Column(Integer, ForeignKey("sentence.pk"), nullable=False)
-    text = relationship(Text, innerjoin=True, backref="sentences", order_by=Sentence.id)
-    sentence = relationship(Sentence, innerjoin=True, backref="text_assocs")
     part_no = Column(Integer)
+    text = relationship(
+        Text, innerjoin=True, backref="sentences", order_by="desc(TextSentence.part_no)"
+    )
+    sentence = relationship(Sentence, innerjoin=True, backref="text_assocs")
 
 
 class SentenceSlice(Base):
